package com.chakracoin.shsm.util;

/**
 * @Author：Cheney
 * @Date：2023/7/20 16:11
 */
import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.DESedeKeySpec;
import javax.crypto.spec.IvParameterSpec;

import static javax.swing.text.html.CSS.Attribute.PADDING;

public class TripleDESUtil {
    private static final String ALGORITHM = "DESede";
    private static final String PADDING = "PKCS5Padding";
    private static final String PADDING7 = "PKCS7Padding";
    private static final String NoPadd = "NoPadding";
    private static final String MODE = "CBC";

    public static byte[] encrypt(byte[] data, byte[] keyBytes) throws Exception {
        DESedeKeySpec spec = new DESedeKeySpec(keyBytes);
        SecretKeyFactory keyFactory = SecretKeyFactory.getInstance(ALGORITHM);
        SecretKey key = keyFactory.generateSecret(spec);

        Cipher cipher = Cipher.getInstance(ALGORITHM);
        cipher.init(Cipher.ENCRYPT_MODE, key);

        return cipher.doFinal(data);
    }

    public static byte[] decrypt(byte[] encryptedData, byte[] keyBytes) throws Exception {
        DESedeKeySpec spec = new DESedeKeySpec(keyBytes);
        SecretKeyFactory keyFactory = SecretKeyFactory.getInstance(ALGORITHM);
        SecretKey key = keyFactory.generateSecret(spec);

        Cipher cipher = Cipher.getInstance(ALGORITHM);
        cipher.init(Cipher.DECRYPT_MODE, key);

        return cipher.doFinal(encryptedData);
    }

    public static byte[] encryptCBC(byte[] data, byte[] keyBytes, byte[] iv) throws Exception {
        DESedeKeySpec spec = new DESedeKeySpec(keyBytes);
        SecretKeyFactory keyFactory = SecretKeyFactory.getInstance(ALGORITHM);
        SecretKey key = keyFactory.generateSecret(spec);

        Cipher cipher = Cipher.getInstance(ALGORITHM + "/" + MODE + "/" + PADDING);
        IvParameterSpec ivSpec = new IvParameterSpec(iv);
        cipher.init(Cipher.ENCRYPT_MODE, key, ivSpec);

        return cipher.doFinal(data);
    }

    public static byte[] decryptCBC(byte[] encryptedData, byte[] keyBytes, byte[] iv) throws Exception {
        DESedeKeySpec spec = new DESedeKeySpec(keyBytes);
        SecretKeyFactory keyFactory = SecretKeyFactory.getInstance(ALGORITHM);
        SecretKey key = keyFactory.generateSecret(spec);

        Cipher cipher = Cipher.getInstance(ALGORITHM + "/" + MODE + "/" + NoPadd);
        IvParameterSpec ivSpec = new IvParameterSpec(iv);
        cipher.init(Cipher.DECRYPT_MODE, key, ivSpec);

        return cipher.doFinal(encryptedData);
    }


    public static void main(String[] args) {
        try {
            String data = "Hello, world!";
            String key = "0123456789abcdef0123456789abcdef0123456789abcdef";
            byte[] encryptedData = encrypt(data.getBytes(), key.getBytes());
            byte[] decryptedData = decrypt(encryptedData, key.getBytes());

            System.out.println("Original Data: " + data);
            System.out.println("Encrypted Data: " + new String(encryptedData));
            System.out.println("Decrypted Data: " + new String(decryptedData));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
