import javax.crypto.Cipher;
import java.nio.charset.StandardCharsets;
import java.security.*;
import java.util.Base64;

public class RSAEncryptionExample {
    public static KeyPair generateKeyPair() throws NoSuchAlgorithmException {
        KeyPairGenerator keyPairGenerator = KeyPairGenerator.getInstance("RSA");
        keyPairGenerator.initialize(2048); // 使用 2048 位的密钥长度
        return keyPairGenerator.generateKeyPair();
    }

    public static byte[] encryptWithPublicKey(String data, PublicKey publicKey) throws Exception {
        Cipher cipher = Cipher.getInstance("RSA");
        cipher.init(Cipher.ENCRYPT_MODE, publicKey);
        return cipher.doFinal(data.getBytes(StandardCharsets.UTF_8));
    }

    public static String decryptWithPrivateKey(byte[] encryptedData, PrivateKey privateKey) throws Exception {
        Cipher cipher = Cipher.getInstance("RSA");
        cipher.init(Cipher.DECRYPT_MODE, privateKey);
        byte[] decryptedBytes = cipher.doFinal(encryptedData);
        return new String(decryptedBytes, StandardCharsets.UTF_8);
    }

    public static void main(String[] args) {
        try {
            // 生成公钥和私钥
            KeyPair keyPair = generateKeyPair();
            PublicKey publicKey = keyPair.getPublic();
            PrivateKey privateKey = keyPair.getPrivate();

            // 要加密的数据
            String data = "Hello, World!";

            // 使用公钥加密数据
            byte[] encryptedData = encryptWithPublicKey(data, publicKey);

            // 将加密后的数据转换为 Base64 编码的字符串（方便在网络传输或存储）
            String encryptedDataString = Base64.getEncoder().encodeToString(encryptedData);
            System.out.println("Encrypted data: " + encryptedDataString);

            // 使用私钥解密数据
            byte[] decryptedBytes = Base64.getDecoder().decode(encryptedDataString);
            String decryptedData = decryptWithPrivateKey(decryptedBytes, privateKey);
            System.out.println("Decrypted data: " + decryptedData);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
