package com.chakracoin.shsm.cert;


import com.sunyard.ssp.BytesUtil;
import org.bouncycastle.asn1.ASN1OctetString;
import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
import org.bouncycastle.asn1.x509.SubjectPublicKeyInfo;
import org.bouncycastle.asn1.x9.X9ECPoint;
import org.bouncycastle.asn1.x9.X9ObjectIdentifiers;
import org.bouncycastle.crypto.params.ECDomainParameters;
import org.bouncycastle.jcajce.provider.asymmetric.util.KeyUtil;
import org.bouncycastle.jce.ECNamedCurveTable;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.jce.spec.ECNamedCurveParameterSpec;
import org.bouncycastle.math.ec.ECCurve;
import org.bouncycastle.math.ec.ECPoint;

import java.math.BigInteger;
import java.security.*;

import static com.chakracoin.shsm.logiclayer.cert.SM2PublicKey.ID_SM2_PUBKEY_PARAM;

public class QPublicKey implements PublicKey {

    private ECPoint q;

    public QPublicKey(ECPoint q) {
        this.q = q;
    }

    @Override
    public String getAlgorithm() {
        return "EC";
    }

    @Override
    public String getFormat() {
        return "PKCS#8";
    }

    @Override
    public byte[] getEncoded() {
        ASN1OctetString p = ASN1OctetString.getInstance(
                new X9ECPoint(q, false).toASN1Primitive());

        // stored curve is null if ImplicitlyCa
        SubjectPublicKeyInfo info = new SubjectPublicKeyInfo(
                new AlgorithmIdentifier(X9ObjectIdentifiers.id_ecPublicKey, ID_SM2_PUBKEY_PARAM),
                p.getOctets());

        byte[]s= KeyUtil.getEncodedSubjectPublicKeyInfo(info);
        return s;
    }


    public static QPublicKey getQPublicKey(String der) throws NoSuchAlgorithmException, NoSuchProviderException {

        if (null == der || !der.startsWith("03420004") ) {
            throw new IllegalArgumentException("公钥格式错误");
        }

        int headLen = 8;
        int len = ( der.length() - 8 ) / 2;

        String strcertPKX = der.substring(headLen, headLen + 64);
        String strcertPKY = der.substring(headLen + len, headLen + len + 64);

        BigInteger biX = new BigInteger(strcertPKX, 16);
        BigInteger biY = new BigInteger(strcertPKY, 16);

        Security.addProvider(new org.bouncycastle.jce.provider.BouncyCastleProvider());                 //载入bc库的支持

        ECNamedCurveParameterSpec spec = ECNamedCurveTable.getParameterSpec("sm2p256v1");       //获取bc库内置曲线参数，曲线名称sm2p256v1

        ECDomainParameters ecDomainParameters = new ECDomainParameters(spec.getCurve(), spec.getG(), spec.getN());


        ECCurve ecurve = ecDomainParameters.getCurve();          //获取曲线对象

        ECPoint q = ecurve.createPoint(biX, biY);        //创建公钥点

        return new QPublicKey( q );
    }
}
