package example;

import com.chakracoin.shsm.logiclayer.BCECUtil;
import com.chakracoin.shsm.logiclayer.SM2Util;
import com.chakracoin.shsm.test.GMBaseTest;
import com.chakracoin.shsm.test.util.FileUtil;
import com.chakracoin.shsm.util.Util;
import org.bouncycastle.crypto.AsymmetricCipherKeyPair;
import org.bouncycastle.crypto.params.ECPrivateKeyParameters;
import org.bouncycastle.crypto.params.ECPublicKeyParameters;
import org.bouncycastle.jcajce.provider.asymmetric.ec.BCECPrivateKey;
import org.bouncycastle.jcajce.provider.asymmetric.ec.BCECPublicKey;
import org.bouncycastle.math.ec.FixedPointCombMultiplier;
import org.bouncycastle.pqc.math.linearalgebra.ByteUtils;
import org.junit.Assert;
import org.junit.Test;

import java.nio.charset.StandardCharsets;
import java.security.KeyPair;
import java.util.Base64;

public class SM2Kit extends GMBaseTest {

    @Test
    public void testKeyPairEncoding() {
        try {
            AsymmetricCipherKeyPair keyPair = SM2Util.generateKeyPairParameter();
            ECPrivateKeyParameters priKey = (ECPrivateKeyParameters) keyPair.getPrivate();
            ECPublicKeyParameters pubKey = (ECPublicKeyParameters) keyPair.getPublic();


            System.out.println(Util.bytes2HexString(priKey.getD().toByteArray()));

            byte[] priKeyPkcs8Der = BCECUtil.convertECPrivateKeyToPKCS8(priKey, pubKey);
            System.out.println("private key pkcs8 der length:" + priKeyPkcs8Der.length);
            System.out.println("private key pkcs8 der:" + ByteUtils.toHexString(priKeyPkcs8Der));
            FileUtil.writeFile("target/ec.pkcs8.pri.der", priKeyPkcs8Der);

            BCECPrivateKey newPriKey = BCECUtil.convertPKCS8ToECPrivateKey(priKeyPkcs8Der);
            System.out.println(newPriKey);


            org.bouncycastle.math.ec.ECPoint q = new FixedPointCombMultiplier().multiply(newPriKey.getParameters().getG(), newPriKey.getD()).normalize();
            ECPublicKeyParameters newPubKey = BCECUtil.createECPublicKeyParameters(
                    q.getAffineXCoord().toBigInteger().toString(16),
                    q.getAffineYCoord().toBigInteger().toString(16),
                    SM2Util.CURVE, SM2Util.DOMAIN_PARAMS
            );


            byte[] data = "123456789".getBytes(StandardCharsets.UTF_8);
            System.out.println("od= " + Util.bytes2HexString(data));
            byte[] sign = SM2Util.sign(newPriKey, data);
            System.out.println(Util.bytes2HexString(sign));
            boolean flag = SM2Util.verify(newPubKey, data, sign);
            if (!flag) {
                Assert.fail("verify failed");
            }


        } catch (Exception ex) {
            ex.printStackTrace();
            Assert.fail();
        }
    }


    private static final String PK_MARK_HEAD = "-----BEGIN ECDSA PUBLIC KEY-----";
    private static final String PK_MARK_TAIL = "-----END ECDSA PUBLIC KEY-----";

    @Test
    public void pk() {
        String data = "592B5E2D7AFEF43552D02A4630A78BBA316A13D0D0C341C595004B2B5CFB25BA11-----BEGIN ECDSA PUBLIC KEY-----MFkwEwYHKoZIzj0CAQYIKoEcz1UBgi0DQgAEAxNwJKcYPJNzAexpc1a5HdpMP1TlZHcja/kAlbq0fSC5m/efIlH28IfV79MFds9yNajGOJffqG2ngIIuRaOooQ==-----END ECDSA PUBLIC KEY-----";
        String pk = "-----BEGIN ECDSA PUBLIC KEY-----MFkwEwYHKoZIzj0CAQYIKoEcz1UBgi0DQgAEAxNwJKcYPJNzAexpc1a5HdpMP1TlZHcja/kAlbq0fSC5m/efIlH28IfV79MFds9yNajGOJffqG2ngIIuRaOooQ==-----END ECDSA PUBLIC KEY-----";
        String sign = "MEUCIQDy+6Ge95Pv5qQWeO8/ooQB5vLp+cEP3KnyvRz3gvGS2wIgWuWZGI42NH1o0Ti2QhJTwAdNnNwvbbxBDwqt7ECtRfQ=";

        if (pk.startsWith(PK_MARK_HEAD) && pk.endsWith(PK_MARK_TAIL)) {
            pk = pk.substring(PK_MARK_HEAD.length(), pk.length() - PK_MARK_TAIL.length());
            pk = pk.trim();
        }

        // 3059301306072A8648CE3D020106082A811CCF5501822D03420004EB1F1EC734D710C016B171C4A0DC5418BD9AE89EEFE8DA7C36D7192B40F8CF9B80EFF7D62F98A9D84BF910B85AD8F2F19D8DC1EE61458A01B7FB21C1850C7290
        System.out.println(pk);


        // EB1F1EC734D710C016B171C4A0DC5418BD9AE89EEFE8DA7C36D7192B40F8CF9B80EFF7D62F98A9D84BF910B85AD8F2F19D8DC1EE61458A01B7FB21C1850C7290
        byte[] pkHex = Base64.getDecoder().decode(pk.getBytes(StandardCharsets.UTF_8));
        String pkDer = Util.bytes2HexString(pkHex);
        pkDer = pkDer.substring(pkDer.indexOf("03420004") + 8);
        String xHex = pkDer.substring(0, 64);
        String yHex = pkDer.substring(64);

        System.out.println("x=" + xHex);
        System.out.println("y=" + yHex);
        ECPublicKeyParameters pubKey = BCECUtil.createECPublicKeyParameters(xHex, yHex, SM2Util.CURVE, SM2Util.DOMAIN_PARAMS);


        byte[] s = Util.decodeBase64( sign );
        System.out.println( Util.bytes2HexString( s ) );


        byte[] d = data.getBytes();
        System.out.println("data=" + Util.bytes2HexString(d) );
        boolean flag = SM2Util.verify(pubKey, data.getBytes(), s);

        System.out.println(flag);

    }

    // 3044
    // 0220
    // 09B9756BA1C3CF45745E0AC9EF5A4AC0B77DB7DC9DA79728B8328AF7DB68689E
    // 0220
    // 4D90EF145087C4488494551ECEE76F8A0DC057C4CCBB10ACD6D2DC3C04F652A4


    @Test
    public void testGenerateBCECKeyPair() {
        try {
            KeyPair keyPair = SM2Util.generateKeyPair();
            ECPrivateKeyParameters priKey = BCECUtil.convertPrivateKeyToParameters((BCECPrivateKey) keyPair.getPrivate());
            ECPublicKeyParameters pubKey = BCECUtil.convertPublicKeyToParameters((BCECPublicKey) keyPair.getPublic());

            byte[] sign = SM2Util.sign(priKey, SRC_DATA);
            boolean flag = SM2Util.verify(pubKey, SRC_DATA, sign);
            if (!flag) {
                Assert.fail("verify failed");
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            Assert.fail();
        }
    }


}
